/***************************************************************************

PROJECT:			CS 297 Deliverable 3

FILE:				controlview.cpp

PURPOSE:			Class implementation for forms view where functionality 
					for controls such as buttons, edit boxes, etc. are
					implemented.  The CControlView class defines the 
					control view for users to interact with.

COMPILER / TOOL:	Visual C++ .NET

PROGRAMMER:			Wallun Chan

START DATE:			4/1/05

***************************************************************************/

#include "stdafx.h"
#include "utility.h"
#include "imagealign.h"
#include "imagealigndoc.h"
#include "edgefinder.h"
#include "ControlView.h"
#include "leftimageview.h"
#include "rightimageview.h"
#include "featureview.h"
#include "directory.h"
#include "convexhull.h"


// CControlView

IMPLEMENT_DYNCREATE(CControlView, CFormView)

CControlView::CControlView() : CFormView(CControlView::IDD)
// Default constructor
{
}

CControlView::~CControlView()
// Destructor
{
}

void CControlView::OnInitialUpdate()
/* PURPOSE:		Initializes GUI controls to their proper states.
*/
{
	CFormView::OnInitialUpdate();
	SetDlgItemText(IDC_SAMPLED_EDIT, "500");
	SetDlgItemText(IDC_ANGULAR_EDIT, "10");
	SetDlgItemText(IDC_RADIAL_EDIT, "8");
	SetDlgItemText(IDC_TEMPLATE_SIGMA_EDIT, "1.5");
	SetDlgItemText(IDC_TEMPLATE_THRESHOLD_EDIT, "12");
	SetDlgItemText(IDC_TEST_SIGMA_EDIT, "1.5");
	SetDlgItemText(IDC_TEST_THRESHOLD_EDIT, "12");
	GetDlgItem(IDC_TEMPLATE_HYS_CHECK)->EnableWindow(FALSE);
	GetDlgItem(IDC_TEMPLATE_THIN_CHECK)->EnableWindow(FALSE);
	GetDlgItem(IDC_TEMPLATE_SHOW_IMAGE_CHECK)->EnableWindow(FALSE);
	GetDlgItem(IDC_TEMPLATE_SHOW_EDGES_CHECK)->EnableWindow(FALSE);
	GetDlgItem(IDC_TEMPLATE_SAMPLED_CHECK)->EnableWindow(FALSE);
	GetDlgItem(IDC_TEMPLATE_DETECT_BUTTON)->EnableWindow(FALSE);
	((CButton*)GetDlgItem(IDC_TEMPLATE_HYS_CHECK))->SetCheck(BST_CHECKED);
	((CButton*)GetDlgItem(IDC_TEMPLATE_THIN_CHECK))->SetCheck(BST_CHECKED);
	((CButton*)GetDlgItem(IDC_TEMPLATE_SHOW_IMAGE_CHECK))->SetCheck(BST_CHECKED);
	((CButton*)GetDlgItem(IDC_TEMPLATE_SHOW_EDGES_CHECK))->SetCheck(BST_CHECKED);
	((CButton*)GetDlgItem(IDC_TEMPLATE_SAMPLED_CHECK))->SetCheck(BST_CHECKED);
	GetDlgItem(IDC_TEST_HYS_CHECK)->EnableWindow(FALSE);
	GetDlgItem(IDC_TEST_THIN_CHECK)->EnableWindow(FALSE);
	GetDlgItem(IDC_TEST_SHOW_IMAGE_CHECK)->EnableWindow(FALSE);
	GetDlgItem(IDC_TEST_SHOW_EDGES_CHECK)->EnableWindow(FALSE);
	GetDlgItem(IDC_TEST_SAMPLED_CHECK)->EnableWindow(FALSE);
	GetDlgItem(IDC_TEST_DETECT_BUTTON)->EnableWindow(FALSE);
	((CButton*)GetDlgItem(IDC_TEST_HYS_CHECK))->SetCheck(BST_CHECKED);
	((CButton*)GetDlgItem(IDC_TEST_THIN_CHECK))->SetCheck(BST_CHECKED);
	((CButton*)GetDlgItem(IDC_TEST_SHOW_IMAGE_CHECK))->SetCheck(BST_CHECKED);
	((CButton*)GetDlgItem(IDC_TEST_SHOW_EDGES_CHECK))->SetCheck(BST_CHECKED);
	((CButton*)GetDlgItem(IDC_TEST_SAMPLED_CHECK))->SetCheck(BST_CHECKED);
	((CButton*)GetDlgItem(IDC_POINT_GROUP_CHECK))->SetCheck(BST_CHECKED);
}

void CControlView::DoDataExchange(CDataExchange* pDX)
// Standard windows initialization
{
	CFormView::DoDataExchange(pDX);
}

BEGIN_MESSAGE_MAP(CControlView, CFormView)
	ON_BN_CLICKED(IDC_TEMPLATE_LOAD_BUTTON, OnBnClickedLoadLeftButton)
	ON_BN_CLICKED(IDC_TEST_LOAD_BUTTON, OnBnClickedLoadRightButton)
	ON_BN_CLICKED(IDC_TEMPLATE_CLEAR_BUTTON, OnBnClickedTemplateClearButton)
	ON_BN_CLICKED(IDC_TEMPLATE_DETECT_BUTTON, OnBnClickedTemplateDetectButton)
	ON_BN_CLICKED(IDC_TEST_CLEAR_BUTTON, OnBnClickedTestClearButton)
	ON_BN_CLICKED(IDC_TEST_DETECT_BUTTON, OnBnClickedTestDetectButton)
	ON_BN_CLICKED(IDC_ALIGN_BUTTON, OnBnClickedAlignButton)
	ON_BN_CLICKED(IDC_TEMPLATE_SHOW_IMAGE_CHECK, OnBnClickedTemplateShowImageCheck)
	ON_BN_CLICKED(IDC_TEMPLATE_SHOW_EDGES_CHECK, OnBnClickedTemplateShowEdgesCheck)
	ON_BN_CLICKED(IDC_TEMPLATE_SAMPLED_CHECK, OnBnClickedTemplateSampledCheck)
	ON_BN_CLICKED(IDC_TEMPLATE_HYS_CHECK, OnBnClickedTemplateHysCheck)
	ON_BN_CLICKED(IDC_TEMPLATE_THIN_CHECK, OnBnClickedTemplateThinCheck)
	ON_BN_CLICKED(IDC_TEST_HYS_CHECK, OnBnClickedTestHysCheck)
	ON_BN_CLICKED(IDC_TEST_THIN_CHECK, OnBnClickedTestThinCheck)
	ON_BN_CLICKED(IDC_TEST_SHOW_IMAGE_CHECK, OnBnClickedTestShowImageCheck)
	ON_BN_CLICKED(IDC_TEST_SHOW_EDGES_CHECK, OnBnClickedTestShowEdgesCheck)
	ON_BN_CLICKED(IDC_TEST_SAMPLED_CHECK, OnBnClickedTestSampledCheck)
	ON_BN_CLICKED(IDC_ERASE_BOXES_BUTTON, OnBnClickedEraseBoxesButton)
	ON_BN_CLICKED(IDC_LOCATE_BUTTON, OnBnClickedLocateButton)
	ON_BN_CLICKED(IDC_FEATURE_RESET_BUTTON, OnBnClickedFeatureResetButton)
	ON_BN_CLICKED(IDC_POINT_GROUP_CHECK, OnBnClickedPointGroupCheck)
	ON_BN_CLICKED(ID_SAVE_TEMPLATE_BUTTON, OnBnClickedSaveTemplateButton)
	ON_BN_CLICKED(ID_SAVE_TEST_BUTTON, OnBnClickedSaveTestButton)
	ON_BN_CLICKED(ID_SAVE_FEATURE_BUTTON, OnBnClickedSaveFeatureButton)
	ON_BN_CLICKED(IDC_LOCATE_REMOVE_OUTLIERS_BUTTON, OnBnClickedLocateRemoveOutliersButton)
END_MESSAGE_MAP()


// CControlView diagnostics

#ifdef _DEBUG
void CControlView::AssertValid() const
{
	CFormView::AssertValid();
}

void CControlView::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}
#endif //_DEBUG

void CControlView::updateTemplateViewState(BOOL updateSample)
/* PURPOSE:		Checks the state of the GUI and based on user
				selections (e.g. check boxes, etc.) will update
				template view to reflect latest state.

   RECEIVES:	Boolean to determine whether pixels should
				be sampled from edge data of template image.
*/
{
	BYTE* image;
	BYTE* edgeMap;
	CString sigmaStr;
	CString thresholdStr;
	CString sampleStr;

	// Get user input values
	GetDlgItemText(IDC_TEMPLATE_SIGMA_EDIT, sigmaStr);
	GetDlgItemText(IDC_TEMPLATE_THRESHOLD_EDIT, thresholdStr);
	GetDlgItemText(IDC_SAMPLED_EDIT, sampleStr);
	double sigma = atof(sigmaStr);
	int threshold = atoi(thresholdStr);
	int oldThreshold = threshold;
	int samples = atoi(sampleStr);

	// Check to see if checkboxes are checked, or not.
	int hysCheck = ((CButton*)GetDlgItem(IDC_TEMPLATE_HYS_CHECK))->GetCheck();
	int thinCheck =	((CButton*)GetDlgItem(IDC_TEMPLATE_THIN_CHECK))->GetCheck();
	int imageCheck = ((CButton*)GetDlgItem(IDC_TEMPLATE_SHOW_IMAGE_CHECK))->GetCheck();
	int edgeCheck = ((CButton*)GetDlgItem(IDC_TEMPLATE_SHOW_EDGES_CHECK))->GetCheck();
	int sampleCheck = ((CButton*)GetDlgItem(IDC_TEMPLATE_SAMPLED_CHECK))->GetCheck();

	CimagealignDoc* doc = (CimagealignDoc*)GetDocument();

	// Release allocated memory for template view image
	delete [] doc->templateImage;

	// Get edge detected template image
	image = doc->templateDetector->get_image();

	/* Get edge detected data based on user selections of 
	   hysteresis, thinning, etc. for template image view. */
	edgeMap = getEdgeMap(doc->templateDetector, hysCheck, thinCheck, sigma, &threshold);

	/* If a zero for threshold is passed in, the edge detection code will
	   estimate a best effort threshold to use. So update new threshold
	   for threshold if this is the case. */
	if (oldThreshold == 0)
	{	thresholdStr.Format("%d", threshold);
		SetDlgItemText(IDC_TEMPLATE_THRESHOLD_EDIT, thresholdStr);
		CString editStr;
		editStr.Format("Template view threshold estimated to be %s.",
			thresholdStr);
		MessageBox(editStr, "Threshold estimation ...", MB_OK | MB_ICONWARNING);
	}

	// If updateSample is true, then (re)sample pixels from edges.
	if (updateSample)
	{	doc->templateSampledPts.clear();
		getSampledPoints(doc->templateSampledPts, edgeMap, 
			doc->templateWidth, doc->templateHeight, samples);
	}

	/* Based on user selections, this function call generates the proper image
	   to be displayed in the template image view. */
	setDisplayImage(doc->templateImage, image, edgeMap, doc->templateSampledPts, 
		doc->templateWidth, doc->templateHeight, imageCheck, edgeCheck, sampleCheck);

	// Redraw template view
	CLeftImageView* view = (CLeftImageView*)doc->getView(RUNTIME_CLASS(CLeftImageView));
	view->Invalidate();
	view->UpdateWindow();
}

void CControlView::updateTestViewState(BOOL updateSample)
/* PURPOSE:		Checks the state of the GUI and based on user
				selections (e.g. check boxes, etc.) will update
				test view to reflect latest state.

   RECEIVES:	Boolean to determine whether pixels should
				be sampled from edge data of test image.
*/
{
	BYTE* image;
	BYTE* edgeMap;
	CString sigmaStr;
	CString thresholdStr;
	CString sampleStr;

	// Get user selections and input in GUI controls
	GetDlgItemText(IDC_TEST_SIGMA_EDIT, sigmaStr);
	GetDlgItemText(IDC_TEST_THRESHOLD_EDIT, thresholdStr);
	GetDlgItemText(IDC_SAMPLED_EDIT, sampleStr);
	double sigma = atof(sigmaStr);
	int threshold = atoi(thresholdStr);
	int oldThreshold = threshold;
	int samples = atoi(sampleStr);

	// Check to see if checkboxes are checked, or not.
	int hysCheck = ((CButton*)GetDlgItem(IDC_TEST_HYS_CHECK))->GetCheck();
	int thinCheck =	((CButton*)GetDlgItem(IDC_TEST_THIN_CHECK))->GetCheck();
	int imageCheck = ((CButton*)GetDlgItem(IDC_TEST_SHOW_IMAGE_CHECK))->GetCheck();
	int edgeCheck = ((CButton*)GetDlgItem(IDC_TEST_SHOW_EDGES_CHECK))->GetCheck();
	int sampleCheck = ((CButton*)GetDlgItem(IDC_TEST_SAMPLED_CHECK))->GetCheck();

	CimagealignDoc* doc = (CimagealignDoc*)GetDocument();

	// Release any memory for test image view
	delete [] doc->testImage;

	// Get edge detected test image
	image = doc->testDetector->get_image();

	/* Get edge detected data based on user selections of hysteresis, 
	   thinning, etc for test image view. */
	edgeMap = getEdgeMap(doc->testDetector, hysCheck, thinCheck, sigma, &threshold);

	/* If a zero for threshold is passed in, the edge detection code will
	   estimate a best effort threshold to use. So update new threshold
	   for threshold if this is the case. */
	if (oldThreshold == 0)
	{	thresholdStr.Format("%d", threshold);
		SetDlgItemText(IDC_TEST_THRESHOLD_EDIT, thresholdStr);
		CString editStr;
		editStr.Format("Test view threshold estimated to be %s.",
			thresholdStr);
		MessageBox(editStr, "Threshold estimation ...", MB_OK | MB_ICONWARNING);
	}

	// If updateSample is true, then (re)sample pixels from edges.
	if (updateSample)
	{	doc->testSampledPts.clear();
		getSampledPoints(doc->testSampledPts, edgeMap, doc->testWidth, doc->testHeight, samples);
	}

	/* Based on user selections, this function call generates the proper image
	   to be displayed in the test image view. */
	setDisplayImage(doc->testImage, image, edgeMap, doc->testSampledPts, 
		doc->testWidth, doc->testHeight, imageCheck, edgeCheck, sampleCheck);
	
	// Redraw template view
	CRightImageView* view = (CRightImageView*)doc->getView(RUNTIME_CLASS(CRightImageView));
	view->Invalidate();
	view->UpdateWindow();
}


// CControlView message handlers

void CControlView::OnBnClickedLoadLeftButton()
/* PURPOSE:		Allows user to select a .pgm image to be displayed
				in template image view.
*/
{
	// Open file display and selection dialog
	CFileDialog dialog(1, 0, 0, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, pgmFilter);
	if (dialog.DoModal() != IDOK)
	{	return;
	}

	int width;
	int height;

	/* Read .pgm image for image dimensions and return a byte string 
	   containing the image data. */
	BYTE* bytes = pgmByteRead(dialog.GetPathName(), width, height);

	CimagealignDoc* doc = (CimagealignDoc*)GetDocument();

	// Release previously allocated memory
	doc->templateDetector->freeImage();
	delete [] doc->templateImage;
	doc->templateSampledPts.clear();
	doc->boxes.clear();

	// Initialize edge detector with image to be edge detected
	doc->templateDetector->setImage(bytes, width, height);

	/* Set document data so that the template view will update to the
	   latest image. */
	doc->templateImage = (BYTE*)rgbFormat(bytes, width * height);
	doc->templateWidth = width;
	doc->templateHeight = height;

	// Set border title
	doc->tempFileName.Format("   (Current image: %s)", 
		GetSubPath(dialog.GetPathName()));

	/* Flag to check if template image is mapped to
	   test image. */
	doc->isMapped = false;

	// Redraw template image view
	CLeftImageView* view = (CLeftImageView*)doc->
		getView(RUNTIME_CLASS(CLeftImageView));
	view->Invalidate();
	view->UpdateWindow();

	// Resize scroll length to fit image
	CSize sizeTotal;
	sizeTotal.cx = (long)(SCROLL_FACTOR * width);
	sizeTotal.cy = (long)(SCROLL_FACTOR * height);
	view->SetScrollSizes(MM_TEXT, sizeTotal);

	// Enable the proper GUI controls
	GetDlgItem(IDC_TEMPLATE_DETECT_BUTTON)->EnableWindow();
	GetDlgItem(IDC_TEMPLATE_HYS_CHECK)->EnableWindow();
	GetDlgItem(IDC_TEMPLATE_THIN_CHECK)->EnableWindow();
	GetDlgItem(IDC_TEMPLATE_SHOW_IMAGE_CHECK)->EnableWindow();
	GetDlgItem(IDC_TEMPLATE_SHOW_EDGES_CHECK)->EnableWindow();
	GetDlgItem(IDC_TEMPLATE_SAMPLED_CHECK)->EnableWindow();
}

void CControlView::OnBnClickedLoadRightButton()
/* PURPOSE:		Allows user to select a .pgm image to be displayed
				in test view.
*/
{
	// Open file display and selection dialog
	CFileDialog dialog(1, 0, 0, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, pgmFilter);
	if (dialog.DoModal() != IDOK)
	{	return;
	}

	int width;
	int height;

	/* Read .pgm image for image dimensions and return a byte string 
	   containing the image data. */
	BYTE* bytes = pgmByteRead(dialog.GetPathName(), width, height);

	CimagealignDoc* doc = (CimagealignDoc*)GetDocument();

	// Release previously allocated memory
	doc->testDetector->freeImage();
	delete [] doc->testImage;
	doc->testSampledPts.clear();

	// Initialize edge detector with image to be edge detected
	doc->testDetector->setImage(bytes, width, height);

	/* Set document data so that the test view will update to the
	   latest image. */
	doc->testImage = (BYTE*)rgbFormat(bytes, width * height);
	doc->testWidth = width;
	doc->testHeight = height;

	// Set border title
	doc->testFileName.Format("   (Current image: %s)", 
		GetSubPath(dialog.GetPathName()));

	/* Flag to check if template image is mapped to
	   test image. */
	doc->isMapped = false;

	// Redraw template image view
	CLeftImageView* view = (CLeftImageView*)doc->getView(RUNTIME_CLASS(CRightImageView));
	view->Invalidate();
	view->UpdateWindow();

	// Resize scroll length to fit image
	CSize sizeTotal;
	sizeTotal.cx = (long)(1.25 * width);
	sizeTotal.cy = (long)(1.25 * height);
	view->SetScrollSizes(MM_TEXT, sizeTotal);

	// Enable the proper GUI controls
	GetDlgItem(IDC_TEST_DETECT_BUTTON)->EnableWindow();
	GetDlgItem(IDC_TEST_HYS_CHECK)->EnableWindow();
	GetDlgItem(IDC_TEST_THIN_CHECK)->EnableWindow();
	GetDlgItem(IDC_TEST_SHOW_IMAGE_CHECK)->EnableWindow();
	GetDlgItem(IDC_TEST_SHOW_EDGES_CHECK)->EnableWindow();
	GetDlgItem(IDC_TEST_SAMPLED_CHECK)->EnableWindow();	
}

void CControlView::OnBnClickedTemplateClearButton()
/* PURPOSE:		Clears template view display and releases memory
				allocated for image that was displayed.
*/
{
	CimagealignDoc* doc = (CimagealignDoc*)GetDocument();

	// Free allocated memory for template image view
	doc->templateDetector->freeImage();
	delete [] doc->templateImage;
	doc->templateImage = 0;
	doc->templateSampledPts.clear();
	doc->boxes.clear();

	// Reset border title
	doc->tempFileName.Empty();

	/* Flag to check if template image is mapped to
	   test image. */
	doc->isMapped = false;

	// Update template window
	CLeftImageView* view = (CLeftImageView*)doc->getView(RUNTIME_CLASS(CLeftImageView));
	view->Invalidate();
	view->UpdateWindow();

	// Reset scroll length
	view->SetScrollSizes(MM_TEXT, CSize(DEFAULT_SCROLL, DEFAULT_SCROLL));

	// Reset GUI controls
	GetDlgItem(IDC_TEMPLATE_DETECT_BUTTON)->EnableWindow(FALSE);
	GetDlgItem(IDC_TEMPLATE_HYS_CHECK)->EnableWindow(FALSE);
	GetDlgItem(IDC_TEMPLATE_THIN_CHECK)->EnableWindow(FALSE);
	GetDlgItem(IDC_TEMPLATE_SHOW_IMAGE_CHECK)->EnableWindow(FALSE);
	GetDlgItem(IDC_TEMPLATE_SHOW_EDGES_CHECK)->EnableWindow(FALSE);
	GetDlgItem(IDC_TEMPLATE_SAMPLED_CHECK)->EnableWindow(FALSE);
	((CButton*)GetDlgItem(IDC_TEMPLATE_HYS_CHECK))->SetCheck(BST_CHECKED);
	((CButton*)GetDlgItem(IDC_TEMPLATE_THIN_CHECK))->SetCheck(BST_CHECKED);
	((CButton*)GetDlgItem(IDC_TEMPLATE_SHOW_IMAGE_CHECK))->SetCheck(BST_CHECKED);
	((CButton*)GetDlgItem(IDC_TEMPLATE_SHOW_EDGES_CHECK))->SetCheck(BST_CHECKED);
	((CButton*)GetDlgItem(IDC_TEMPLATE_SAMPLED_CHECK))->SetCheck(BST_CHECKED);
}

void CControlView::OnBnClickedTemplateDetectButton()
/* PURPOSE:		Invokes edge detection and the display of edge
				detection data for template image view.
*/
{
	// Update template view to reflect latest image
	updateTemplateViewState();

	// Set flag to indicate that images need to be mapped
	((CimagealignDoc*)GetDocument())->isMapped = false;
}

void CControlView::OnBnClickedTestClearButton()
/* PURPOSE:		Clears test view display and releases memory
				allocated for image that was displayed.
*/
{
	CimagealignDoc* doc = (CimagealignDoc*)GetDocument();

	// Free allocated memory for template image view
	doc->testDetector->freeImage();
	delete [] doc->testImage;
	doc->testImage = 0;
	doc->testSampledPts.clear();

	// Reset border title
	doc->testFileName.Empty();

	/* Flag to check if template image is mapped to
	   test image. */
	doc->isMapped = false;

	// Update template window
	CRightImageView* view = (CRightImageView*)doc->getView(RUNTIME_CLASS(CRightImageView));
	view->Invalidate();
	view->UpdateWindow();

	// Reset scroll length
	view->SetScrollSizes(MM_TEXT, CSize(DEFAULT_SCROLL, DEFAULT_SCROLL));

	// Reset GUI controls
	GetDlgItem(IDC_TEST_DETECT_BUTTON)->EnableWindow(FALSE);
	GetDlgItem(IDC_TEST_HYS_CHECK)->EnableWindow(FALSE);
	GetDlgItem(IDC_TEST_THIN_CHECK)->EnableWindow(FALSE);
	GetDlgItem(IDC_TEST_SHOW_IMAGE_CHECK)->EnableWindow(FALSE);
	GetDlgItem(IDC_TEST_SHOW_EDGES_CHECK)->EnableWindow(FALSE);
	GetDlgItem(IDC_TEST_SAMPLED_CHECK)->EnableWindow(FALSE);
	((CButton*)GetDlgItem(IDC_TEST_HYS_CHECK))->SetCheck(BST_CHECKED);
	((CButton*)GetDlgItem(IDC_TEST_THIN_CHECK))->SetCheck(BST_CHECKED);
	((CButton*)GetDlgItem(IDC_TEST_SHOW_IMAGE_CHECK))->SetCheck(BST_CHECKED);
	((CButton*)GetDlgItem(IDC_TEST_SHOW_EDGES_CHECK))->SetCheck(BST_CHECKED);
	((CButton*)GetDlgItem(IDC_TEST_SAMPLED_CHECK))->SetCheck(BST_CHECKED);	
}

void CControlView::OnBnClickedTestDetectButton()
/* PURPOSE:		Invokes edge detection and the display of edge
				detection data for test image view.
*/
{
	// Update test view to reflect latest image
	updateTestViewState();

	/* Flag to check if template image is mapped to
	   test image. */
	((CimagealignDoc*)GetDocument())->isMapped = false;
}

void CControlView::OnBnClickedAlignButton()
/* PURPOSE:		This function takes the sampled points of edge data
				from the template image and aligns it with the sampled
				points of the test image.  Based on this alignment,
				the rest of the edge data is mapped over to the 
				test image.  The mapped image is then displayed.
*/
{
	CimagealignDoc* doc = (CimagealignDoc*)GetDocument();

	// Check to see if images have been loaded before aligning
	if (doc->templateImage == 0)
	{	MessageBox("Template image not loaded.", 
			"Load image ...", MB_OK | MB_ICONWARNING);
		return;
	}
	if (doc->testImage == 0)
	{	MessageBox("Test image not loaded.", 
			"Load image ...", MB_OK | MB_ICONWARNING);
		return;
	}
	if (doc->templateSampledPts.size() == 0)
	{	MessageBox("Template image not edge detected.", 
			"Load image ...", MB_OK | MB_ICONWARNING);
		return;
	}
	if (doc->testSampledPts.size() == 0)
	{	MessageBox("Test image not edge detected.", 
			"Load image ...", MB_OK | MB_ICONWARNING);
		return;
	}

	// Get user selected and input data from GUI.
	CString sampNumStr;
	CString angBinStr;
	CString radialBinStr;
	GetDlgItemText(IDC_SAMPLED_EDIT, sampNumStr);
	GetDlgItemText(IDC_ANGULAR_EDIT, angBinStr);
	GetDlgItemText(IDC_RADIAL_EDIT, radialBinStr);
	int samples = atoi(sampNumStr);
	int angBinNum = atoi(angBinStr);
	int radialBinNum = atoi(radialBinStr);

	Map tempContext;
	Map testContext;

	// Make copies of sampled points
	VecIntPr tempPts = doc->templateSampledPts;
	VecIntPr testPts = doc->testSampledPts;

	/* Generate shape context data for sampled pixels from edges
	   of template and test images. */
	setShapeContextPair(tempContext, testContext, 
		tempPts, testPts, radialBinNum, angBinNum);

	// Update progress bar
	SendDlgItemMessage(IDC_PROGRESS, PBM_SETPOS, 25, 0);

	// Find point-to-point correspondence between sampled points
	matchPoints(tempContext, testContext, tempPts, testPts);
	
	// Update progress bar
	SendDlgItemMessage(IDC_PROGRESS, PBM_SETPOS, 75, 0);

	// Map template sampled edge points to test image space
	doc->tps.setPoints(testPts, tempPts);

	// Flag to indicate that template and test images have been mapped
	doc->isMapped = true;

	// Update progress bar
	SendDlgItemMessage(IDC_PROGRESS, PBM_SETPOS, 100, 0);

	MessageBox("Template image mapped to test image.", 
		"Images are mapped ...", MB_OK | MB_ICONWARNING);

	// Reset progress bar
	SendDlgItemMessage(IDC_PROGRESS, PBM_SETPOS, 0, 0);
}

void CControlView::OnBnClickedTemplateShowImageCheck()
/* PURPOSE:		To show or not show loaded image in template
				image view.
*/
{
	/* Update template view to reflect latest image 
	   without point sampling. */
	updateTemplateViewState(FALSE);
}

void CControlView::OnBnClickedTemplateShowEdgesCheck()
/* PURPOSE:		To show or not show edge data of image in template
				image view.
*/
{
	/* Update template view to reflect latest image 
	   without point sampling. */
	updateTemplateViewState(FALSE);
}

void CControlView::OnBnClickedTemplateSampledCheck()
/* PURPOSE:		To show or not show sampled pixels of edges of image 
				in template image view.
*/
{
	/* Update template view to reflect latest image 
	   without point sampling. */
	updateTemplateViewState(FALSE);
}

void CControlView::OnBnClickedTemplateHysCheck()
/* PURPOSE:		To show or not show hysteresis applied edge data of 
				image in template image view.
*/
{
	/* Update template view to reflect latest image 
	   without point sampling. */
	updateTemplateViewState(FALSE);
}

void CControlView::OnBnClickedTemplateThinCheck()
/* PURPOSE:		To show or not show thinned edge data of 
				image in template image view.
*/
{
	/* Update template view to reflect latest image 
	   without point sampling. */
	updateTemplateViewState(FALSE);
}

void CControlView::OnBnClickedTestHysCheck()
/* PURPOSE:		To show or not show hysteresis applied edge data of 
				image in test image view.
*/
{
	/* Update test view to reflect latest image 
	   without point sampling. */
	updateTestViewState(FALSE);
}

void CControlView::OnBnClickedTestThinCheck()
/* PURPOSE:		To show or not show thinned edge data of 
				image in test image view.
*/
{
	/* Update test view to reflect latest image 
	   without point sampling. */
	updateTestViewState(FALSE);
}

void CControlView::OnBnClickedTestShowImageCheck()
/* PURPOSE:		To show or not show loaded image in test
				image view.
*/
{
	/* Update test view to reflect latest image 
	   without point sampling. */
	updateTestViewState(FALSE);
}

void CControlView::OnBnClickedTestShowEdgesCheck()
/* PURPOSE:		To show or not show edge data of image in test
				image view.
*/
{
	/* Update test view to reflect latest image 
	   without point sampling. */
	updateTestViewState(FALSE);
}

void CControlView::OnBnClickedTestSampledCheck()
/* PURPOSE:		To show or not show sampled pixels of edges of image 
				in test image view.
*/
{
	/* Update test view to reflect latest image 
	   without point sampling. */
	updateTestViewState(FALSE);
}

void CControlView::OnBnClickedEraseBoxesButton()
/* PURPOSE:		Erases rectangles that are drawn in template 
				image view.
*/
{
	CimagealignDoc* doc = (CimagealignDoc*)GetDocument();
	doc->boxes.clear();
	doc->ptGroup.clear();
	CLeftImageView* view = (CLeftImageView*)doc->
		getView(RUNTIME_CLASS(CLeftImageView));
	view->Invalidate();
	view->UpdateWindow();
}

void CControlView::featureLocate(BOOL showOutliers)
{
	CimagealignDoc* doc = (CimagealignDoc*)GetDocument();

	/* Used to store coordinates of all user selected points in
	   template view. */
	VecIntPr mapped;

	doc->ptGroup.clear();
	doc->centroids.clear();

	/* Goes through all points contained within rectangles
	   drawn by user and stores them in a list to be displayed
	   in feature locate view. */
	unsigned n;
	for (n = 0; n < doc->boxes.size(); n++)
	{	int left = doc->boxes[n].first.first;
		int top	= doc->boxes[n].first.second;
		int right = doc->boxes[n].second.first;
		int bottom = doc->boxes[n].second.second;

		/* Used to store groups of user selected sampled points in
		   template image.  Difference between this and "mapped" is
		   that "mapped" is used to show the mapped points, whereas 
		   "pts" is used to draw the convex hull around the points 
		   stored in "map". */
		VecIntPr pts;

		// Checks for points within user-specified rectangle
		unsigned i;
		for (i = 0; i < doc->templateSampledPts.size(); i++)
		{	IntPr pt = doc->templateSampledPts[i];
			if (pt.first > left && pt.first < right &&
				pt.second > top - BORDER_HEIGHT && 
				pt.second < bottom - BORDER_HEIGHT)
			{	pts.push_back(pt);
			}
		}
		doc->tps.mapPoints(pts);

		// Show all points, including extreme outliers
		if (showOutliers)
		{	// Store centroids of groups of points
			doc->centroids.push_back(centroid(pts));

			/* Store gift-wrapped points to be displayed in feature
			   locate view. */
			doc->ptGroup.push_back(convexHull(pts));
			mapped.insert(mapped.end(), pts.begin(), pts.end());
		}
		/* Remove outliers that are not within a specified number
		   of standard deviations from the mean. */
		else
		{	VecIntPr adjPts = removeOutliers(pts);
			doc->centroids.push_back(centroid(adjPts));
			doc->ptGroup.push_back(convexHull(adjPts));
			mapped.insert(mapped.end(), adjPts.begin(), adjPts.end());
		}
	}

	doc->featureWidth = doc->testWidth;
	doc->featureHeight = doc->testHeight;
	delete [] doc->featureImage;

	/* Overlay mapped points onto test image edge detected data and
	   save to document data to be displayed in feature locate view. */
	doc->featureImage = (BYTE*)overlay(doc->testDetector->get_edge_map(), 
		doc->testWidth, doc->testHeight, mapped, BLUE, YLW);
}

void CControlView::OnBnClickedLocateButton()
/* PURPOSE:		Based on rectangles drawn in the template image
				by user, this function maps the sampled points
				to the test image space, that are contained 
				within these rectangles and displays them 
				in the feature locate view.
*/
{
	CimagealignDoc* doc = (CimagealignDoc*)GetDocument();
	if (!doc->isMapped)
	{	MessageBox("Images have not been mapped.", 
			"Map image ...", MB_OK | MB_ICONWARNING);
		return;
	}

	/* Locate user-selected features on template image to
	   be displayed in feature locate view; show outliers. */
	featureLocate(TRUE);

	// Update feature locate view
	CFeatureView* view = (CFeatureView*)doc->getView(RUNTIME_CLASS(CFeatureView));
	view->Invalidate();
	view->UpdateWindow();

	// Resize scroll length to fit mapped image
	CSize sizeTotal;
	sizeTotal.cx = (long)(SCROLL_FACTOR * doc->featureWidth);
	sizeTotal.cy = (long)(SCROLL_FACTOR * doc->featureHeight);
	view->SetScrollSizes(MM_TEXT, sizeTotal);
}

void CControlView::OnBnClickedLocateRemoveOutliersButton()
{
	CimagealignDoc* doc = (CimagealignDoc*)GetDocument();
	if (!doc->isMapped)
	{	MessageBox("Images have not been mapped.", 
			"Map image ...", MB_OK | MB_ICONWARNING);
		return;
	}

	/* Locate user-selected features on template image to
	   be displayed in feature locate view; remove outliers. */
	featureLocate(FALSE);

	// Update feature locate view
	CFeatureView* view = (CFeatureView*)doc->getView(RUNTIME_CLASS(CFeatureView));
	view->Invalidate();
	view->UpdateWindow();

	// Resize scroll length to fit mapped image
	CSize sizeTotal;
	sizeTotal.cx = (long)(SCROLL_FACTOR * doc->featureWidth);
	sizeTotal.cy = (long)(SCROLL_FACTOR * doc->featureHeight);
	view->SetScrollSizes(MM_TEXT, sizeTotal);
}

void CControlView::OnBnClickedFeatureResetButton()
/* PURPOSE:		Clears display and releases memory allocated for
				image displayed in feature locate view.
*/
{
	CimagealignDoc* doc = (CimagealignDoc*)GetDocument();
	delete [] doc->featureImage;
	doc->featureImage = 0;
	doc->ptGroup.clear();
	CFeatureView* view = (CFeatureView*)doc->getView(RUNTIME_CLASS(CFeatureView));
	view->Invalidate();
	view->UpdateWindow();
	view->SetScrollSizes(MM_TEXT, CSize(DEFAULT_SCROLL, DEFAULT_SCROLL));	
}

void CControlView::OnBnClickedPointGroupCheck()
/* PURPOSE:		Based on user selection, this function controls the display
				of the grouping of selected points in the template image
				view that are mapped over to the test view shown in
				the feature locate view.
*/
{
	CimagealignDoc* doc = (CimagealignDoc*)GetDocument();
	if (doc->featureImage == 0)
	{	MessageBox("Feature(s) not located.", 
			"No features ...", MB_OK | MB_ICONWARNING);
		return;
	}

	/* If group checkbox is checked, show convex hull of mapped points
	   in feature locate view. */
	doc->showGrouping = ((CButton*)GetDlgItem(IDC_POINT_GROUP_CHECK))->GetCheck();

	// Update feature locate view
	CFeatureView* view = (CFeatureView*)doc->getView(RUNTIME_CLASS(CFeatureView));
	view->Invalidate();
	view->UpdateWindow();
}

void CControlView::OnBnClickedSaveTemplateButton()
/* PURPOSE:		Allows user to save displayed image in template
				image view as .bmp image file.
*/
{
	CimagealignDoc* doc = (CimagealignDoc*)GetDocument();
	if (doc->templateImage == 0)
	{	MessageBox("Template image has not been loaded.", 
			"Load template image ...", MB_OK | MB_ICONWARNING);
		return;
	}
	CFileDialog dialog(0, 0, 0, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, bmpFilter);
	if (dialog.DoModal() != IDOK)
	{	return;
	}
	imageOutput(dialog.GetPathName(), doc->templateImage,
		doc->templateWidth, doc->templateHeight);
}

void CControlView::OnBnClickedSaveTestButton()
/* PURPOSE:		Allows user to save displayed image in test
				image view as .bmp image file.
*/
{
	CimagealignDoc* doc = (CimagealignDoc*)GetDocument();
	if (doc->testImage == 0)
	{	MessageBox("Test image has not been loaded.", 
			"Load test image ...", MB_OK | MB_ICONWARNING);
		return;
	}
	CFileDialog dialog(0, 0, 0, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, bmpFilter);
	if (dialog.DoModal() != IDOK)
	{	return;
	}
	imageOutput(dialog.GetPathName(), doc->testImage, 
		doc->testWidth, doc->testHeight);
}

void CControlView::OnBnClickedSaveFeatureButton()
/* PURPOSE:		Allows user to save displayed image in feature
				locate image view as .bmp image file.
*/
{
	CimagealignDoc* doc = (CimagealignDoc*)GetDocument();
	if (doc->featureImage == 0)
	{	MessageBox("Features have not been located.", 
			"Features not located ...", MB_OK | MB_ICONWARNING);
		return;
	}
	CFileDialog dialog(0, 0, 0, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, bmpFilter);
	if (dialog.DoModal() != IDOK)
	{	return;
	}
	imageOutput(dialog.GetPathName(), doc->featureImage,
		doc->featureWidth, doc->featureHeight);
}