/***************************************************************************

PROJECT:			CS 297 Deliverable 3

FILE:				featureview.cpp

PURPOSE:			Class implementation for view that displays 
					user selected sampled points that are mapped 
					from template view to test view. 

COMPILER / TOOL:	Visual C++ .NET

PROGRAMMER:			Wallun Chan

START DATE:			4/1/05

***************************************************************************/

#include "stdafx.h"
#include "imagealign.h"
#include "FeatureView.h"
#include "imagealigndoc.h"
#include "utility.h"


// CFeatureView

IMPLEMENT_DYNCREATE(CFeatureView, CScrollView)

CFeatureView::CFeatureView()
// Default constructor
{
}

CFeatureView::~CFeatureView()
// Destructor
{
}


BEGIN_MESSAGE_MAP(CFeatureView, CScrollView)
END_MESSAGE_MAP()


// CFeatureView drawing

void CFeatureView::OnInitialUpdate()
/* PURPOSE:		Initializes view during startup
*/
{
	CScrollView::OnInitialUpdate();

	// Scroll length initialization
	CSize sizeTotal;
	sizeTotal.cx = sizeTotal.cy = DEFAULT_SCROLL;
	SetScrollSizes(MM_TEXT, sizeTotal);
}

void CFeatureView::OnDraw(CDC* pDC)
/* PURPOSE:		Called when client window needs to be redrawn.

   RECEIVES:	Device context to client window for drawing.

   REMARKS:		Draws image border, paints background black,
				draws current image with user selected points 
				mapped from template view, and draws convex
				hull around them.
*/
{
	CRect rect;
	CRect rcClient;

	CimagealignDoc* doc = (CimagealignDoc*)GetDocument();

	// Get client window area in pixel dimensions
	GetClientRect(rcClient);

	// Get system defined colors
	COLORREF crLight = GetSysColor(COLOR_BTNHIGHLIGHT);
	COLORREF crShadow = GetSysColor(COLOR_BTNSHADOW);
	COLORREF crBtnFace = GetSysColor(COLOR_BTNFACE);
	pDC->SetBkMode(TRANSPARENT);
	CGdiObject *pOldFont = pDC->SelectStockObject(ANSI_VAR_FONT);
	rect = rcClient;

	// Draw title border and set title text
	rect.bottom = rect.top + BORDER_HEIGHT;
	rect.right = BORDER_LIMIT;
	pDC->FillSolidRect(rect, crBtnFace);
	pDC->Draw3dRect(rect, crLight, crShadow);
	CString borderTitle = "     Feature Locate View - ";
	borderTitle += "( Mapped user-selected points from template view )";
	pDC->DrawText(borderTitle, rect, DT_SINGLELINE | DT_VCENTER);

	// Fill client area as black
	rect.top = rect.bottom;
	rect.bottom = BORDER_LIMIT;
	pDC->FillSolidRect(rect, BLK);
	pDC->SelectObject(pOldFont);

	int width = doc->testWidth;
	int height = doc->testHeight;

	if (doc->featureImage == 0)
		return;
	
	// Create device compatible bitmap
	CBitmap bitmap;
	bitmap.CreateCompatibleBitmap(pDC, width, height);

	/* Create compatible device context as storage to
	   blit image data from. */
	CDC cdc;
	cdc.CreateCompatibleDC(pDC);

	// Initialize bitmap with feature located image data and dimensions
	bitmap.SetBitmapBits(width * height * sizeof(DWORD), doc->featureImage);
	CBitmap* oldBitmap = cdc.SelectObject(&bitmap);

	// Blit image to client window
	pDC->BitBlt(0, BORDER_HEIGHT, width, height, &cdc, 0, 0, SRCCOPY);

	// Display Cross hairs and convex hulls in white
	pDC->SelectStockObject(WHITE_PEN);

	int i;
	int j;

	// Draw cross-hairs representing centroids of groups of points
	for (i = 0; i < (int)doc->centroids.size(); i++)
	{	IntPr pt = doc->centroids[i];
		pDC->MoveTo(pt.first, pt.second + BORDER_HEIGHT);
		pDC->LineTo(pt.first + CROSS_HAIR_SIZE, pt.second + 
			CROSS_HAIR_SIZE + BORDER_HEIGHT);
		pDC->MoveTo(pt.first, pt.second + BORDER_HEIGHT);
		pDC->LineTo(pt.first - CROSS_HAIR_SIZE, pt.second + 
			CROSS_HAIR_SIZE + BORDER_HEIGHT);
		pDC->MoveTo(pt.first, pt.second + BORDER_HEIGHT);
		pDC->LineTo(pt.first + CROSS_HAIR_SIZE, pt.second - 
			CROSS_HAIR_SIZE + BORDER_HEIGHT);
		pDC->MoveTo(pt.first, pt.second + BORDER_HEIGHT);
		pDC->LineTo(pt.first - CROSS_HAIR_SIZE, pt.second - 
			CROSS_HAIR_SIZE + BORDER_HEIGHT);
	}

	if (doc->showGrouping)
	{	// Draw convex hull around user selected sampled points
		for (i = 0; i < (int)doc->ptGroup.size(); i++)
		{	VecIntPr group = doc->ptGroup[i];
			int n = (int)group.size();
			if (n >= 3)
			{	for (j = 0; j < n; j++)
				{	if (j == n - 1)
					{	pDC->MoveTo(group[0].first, 
							group[0].second + BORDER_HEIGHT);
						pDC->LineTo(group[n - 1].first, 
							group[n - 1].second + BORDER_HEIGHT);
					}
					else
					{	pDC->MoveTo(group[j].first, 
							group[j].second + BORDER_HEIGHT);
						pDC->LineTo(group[j + 1].first, 
							group[j + 1].second + BORDER_HEIGHT);
					}
				}
			}
		}
	}

	pDC->SelectObject(oldBitmap);
}


// CFeatureView diagnostics

#ifdef _DEBUG
void CFeatureView::AssertValid() const
{
	CScrollView::AssertValid();
}

void CFeatureView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}
#endif //_DEBUG


// CFeatureView message handlers
