/***************************************************************************

PROJECT:			CS 297 Deliverable 3

FILE:				leftimageview.cpp

PURPOSE:			Class implementation for template view where user  
					may select and display an image to be mapped to  
					the test image.

COMPILER / TOOL:	Visual C++ .NET

PROGRAMMER:			Wallun Chan

START DATE:			4/1/05

***************************************************************************/

#include "stdafx.h"
#include "imagealign.h"
#include "LeftImageView.h"
#include "imagealigndoc.h"
#include "edgefinder.h"


// CLeftImageView

IMPLEMENT_DYNCREATE(CLeftImageView, CScrollView)

CLeftImageView::CLeftImageView()
// Default constructor
{
}

CLeftImageView::~CLeftImageView()
// Destructor
{
}


BEGIN_MESSAGE_MAP(CLeftImageView, CScrollView)
	ON_WM_LBUTTONDOWN()
END_MESSAGE_MAP()


// CLeftImageView drawing

void CLeftImageView::OnInitialUpdate()
/* PURPOSE:		Initializes view during startup
*/
{
	CScrollView::OnInitialUpdate();

	// Default scroll initialization
	CSize sizeTotal;
	sizeTotal.cx = sizeTotal.cy = DEFAULT_SCROLL;
	SetScrollSizes(MM_TEXT, sizeTotal);

	// Set border title
	((CimagealignDoc*)GetDocument())->tempBorderTitle = 
		"     Template Image View";
}

void CLeftImageView::OnDraw(CDC* pDC)
/* PURPOSE:		Called when client window needs to be redrawn.

   RECEIVES:	Device context to client window for drawing.

   REMARKS:		Draws image border, paints background black,
				draws current image with edge detected data
				and sampled points, and draws user selected
				rectangles around sampled points.
*/
{
	CRect rect;
	CRect rcClient;

	CimagealignDoc* doc = (CimagealignDoc*)GetDocument();

	// Get client window area in pixel dimensions
	GetClientRect(rcClient);
	
	// Get system defined colors
	COLORREF crLight = GetSysColor(COLOR_BTNHIGHLIGHT);
	COLORREF crShadow = GetSysColor(COLOR_BTNSHADOW);
	COLORREF crBtnFace = GetSysColor(COLOR_BTNFACE);
	pDC->SetBkMode(TRANSPARENT);
	CGdiObject *pOldFont = pDC->SelectStockObject(ANSI_VAR_FONT);
	rect = rcClient;

	// Draw title border and set title text
	rect.bottom = rect.top + BORDER_HEIGHT;
	rect.right = BORDER_LIMIT;
	pDC->FillSolidRect(rect, crBtnFace);
	pDC->Draw3dRect(rect, crLight, crShadow);
	pDC->DrawText(doc->tempBorderTitle + doc->tempFileName, 
		rect, DT_SINGLELINE | DT_VCENTER);

	// Fill client area as black
	rect.top = rect.bottom;
	rect.bottom = BORDER_LIMIT;
	pDC->FillSolidRect(rect, BLK);
	pDC->SelectObject(pOldFont);

	int width = doc->templateWidth;
	int height = doc->templateHeight;
	
	// Create device compatible bitmap
	CBitmap bitmap;
	bitmap.CreateCompatibleBitmap(pDC, width, height);

	/* Create compatible device context as storage to
	   blit image data from. */
	CDC cdc;
	cdc.CreateCompatibleDC(pDC);

	// Initialize bitmap with template image data and dimensions
	bitmap.SetBitmapBits(width * height * sizeof(DWORD), doc->templateImage);
	CBitmap* oldBitmap = cdc.SelectObject(&bitmap);

	// Display bitmap in client window
	pDC->BitBlt(0, BORDER_HEIGHT, width, height, &cdc, 0, 0, SRCCOPY);

	CBrush brush(CYAN);
	pDC->SelectObject(&brush);

	// Draw rectangle around user selected sampled points
	int i;
	for (i = 0; i < (int)doc->boxes.size(); i++)
	{	Box box = doc->boxes[i];
		IntPr pt1 = box.first;
		IntPr pt2 = box.second;
		CRect cRect(pt1.first, pt1.second, pt2.first, pt2.second);
		pDC->FrameRect(&cRect, &brush);
	}

	pDC->SelectObject(oldBitmap);
}

void CLeftImageView::OnLButtonDown(UINT nFlags, CPoint point)
/* PURPOSE:		Allows user to draw rectangles around sampled
				points in the template view that are then
				mapped over to the test view.  Points
				inside rectangle are then high-lighted
				in feature locate view.

   RECEIVES:	Coordinates of clicked cursor when in client window.
*/
{
	CimagealignDoc* doc = (CimagealignDoc*)GetDocument();
	if (doc->templateImage == 0)
	{	return;
	}

	/* This index keeps track of whether user has clicked twice in
	   client window to draw rectangle. */
	int index = doc->pointCount % 2;

	// Gets current position of scroll bar in pixel dimension
	CPoint scrollPos = GetScrollPosition();

	int width = doc->templateWidth;
	int height = doc->templateHeight;

	/* Add length scrolled to current position of cursor 
	   in client window. */
	int posX = point.x + scrollPos.x;
	int posY = point.y + scrollPos.y;

	// If cursor is outside image dimensions, alert user.
	if (posX >= width || posY >= height + BORDER_HEIGHT || posY <= BORDER_HEIGHT)
	{	MessageBox("Cursor outside of image.", 
			"Outside image ...", MB_OK | MB_ICONWARNING);
		return;
	}

	// Store user-clicked position in client window
	doc->boxBoundary[index].first = posX;
	doc->boxBoundary[index].second = posY;

	/* This converts point coordinates so that user may draw
	   rectangle in any orientation (e.g. from left to right,
	   right to left, top to bottom, or bottom to top). */
	if (index == 1)
	{	int x1 = doc->boxBoundary[0].first;
		int y1 = doc->boxBoundary[0].second;
		int x2 = doc->boxBoundary[1].first;
		int y2 = doc->boxBoundary[1].second;
		int minX = __min(x1, x2);
		int minY = __min(y1, y2);
		int maxX = __max(x1, x2);
		int maxY = __max(y1, y2);

		// Store coordinates of rectangles in vector
		doc->boxes.push_back(Box(IntPr(minX, minY), IntPr(maxX, maxY)));
		Invalidate();
		UpdateWindow();
	}
	doc->pointCount++;
}


// CLeftImageView diagnostics

#ifdef _DEBUG
void CLeftImageView::AssertValid() const
{
	CScrollView::AssertValid();
}

void CLeftImageView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}
#endif //_DEBUG


// CLeftImageView message handlers
