/***************************************************************************

PROJECT:			CS 297 Deliverable 3

FILE:				mainfrm.cpp

PURPOSE:			Class implementation for frame window to 
					attach splitter windows and child views to.

COMPILER / TOOL:	Visual C++ .NET

PROGRAMMER:			Wallun Chan

START DATE:			4/1/05

***************************************************************************/

#include "stdafx.h"
#include "imagealign.h"
#include "leftimageview.h"
#include "rightimageview.h"
#include "controlview.h"
#include "featureview.h"

#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	ON_WM_CREATE()
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};


// CMainFrame construction/destruction

CMainFrame::CMainFrame()
// Default constructor
{
}

CMainFrame::~CMainFrame()
// Destructor
{
}


int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
/* PURPOSE:		Initializes and aligns toolbar.
*/
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!m_wndToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}
	// TODO: Delete these three lines if you don't want the toolbar to be dockable
	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);
	SetTitle("Image Align / Feature Locate Program");

	return 0;
}

BOOL CMainFrame::OnCreateClient(LPCREATESTRUCT lpcs, CCreateContext* pContext)
/* PURPOSE:		Creates and initializes the splitter windows and child views.
*/
{
	if (!m_wndSplitterLR.CreateStatic(this, 1, 2))
	{	TRACE0("Failed to create left/right splitter window.\n");
		return FALSE;
	}
	if (!m_wndSplitterLR.CreateView(0, 0, RUNTIME_CLASS(CControlView), CSize(), pContext))
	{	TRACE0("Failed to create control view.\n");
		return FALSE;
	}
	if (!m_wndSplitterUD.CreateStatic(&m_wndSplitterLR, 2, 1, 
		WS_CHILD | WS_VISIBLE, m_wndSplitterLR.IdFromRowCol(0, 1)))
	{	TRACE0("Failed to create up/down splitter window.\n");
		return FALSE;
	}
	if (!m_wndSplitterPIC.CreateStatic(&m_wndSplitterUD, 1, 2, 
		WS_CHILD | WS_VISIBLE, m_wndSplitterUD.IdFromRowCol(0, 0)))
	{	TRACE0("Failed to create picture splitter window.\n");
		return FALSE;
	}
	if (!m_wndSplitterPIC.CreateView(0, 0, RUNTIME_CLASS(CLeftImageView), CSize(), pContext))
	{	TRACE0("Failed to create template view.\n");
		return FALSE;
	}
	if (!m_wndSplitterPIC.CreateView(0, 1, RUNTIME_CLASS(CRightImageView), CSize(), pContext))
	{	TRACE0("Failed to create test view.\n");
		return FALSE;
	}
	if (!m_wndSplitterUD.CreateView(1, 0, RUNTIME_CLASS(CFeatureView), CSize(), pContext)) 
	{	TRACE0("Failed to create feature view.\n");
		return FALSE;
	}
	m_wndSplitterLR.SetColumnInfo(0, 300, 0);
	m_wndSplitterUD.SetRowInfo(0, 450, 0);
	m_wndSplitterPIC.SetColumnInfo(0, 550, 0);
	return TRUE;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
// Windows initialization routine
{
	if( !CFrameWnd::PreCreateWindow(cs) )
		return FALSE;
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return TRUE;
}


// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG


// CMainFrame message handlers

