/***************************************************************************

PROJECT:			CS 297 Deliverable 3

FILE:				rightimageview.cpp

PURPOSE:			Class implementation for test image view where user  
					may select and display an image that provides  
					control points for the template image to be 
					mapped to.

COMPILER / TOOL:	Visual C++ .NET

PROGRAMMER:			Wallun Chan

START DATE:			4/1/05

***************************************************************************/

#include "stdafx.h"
#include "imagealign.h"
#include "RightImageView.h"
#include "imagealigndoc.h"


// CRightImageView

IMPLEMENT_DYNCREATE(CRightImageView, CScrollView)

CRightImageView::CRightImageView()
// Default constructor
{
}

CRightImageView::~CRightImageView()
// Destructor
{
}


BEGIN_MESSAGE_MAP(CRightImageView, CScrollView)
END_MESSAGE_MAP()


// CRightImageView drawing

void CRightImageView::OnInitialUpdate()
{
	CScrollView::OnInitialUpdate();

	// Default scroll initialization
	CSize sizeTotal;
	sizeTotal.cx = sizeTotal.cy = DEFAULT_SCROLL;
	SetScrollSizes(MM_TEXT, sizeTotal);

	// Set border title
	((CimagealignDoc*)GetDocument())->testBorderTitle = "     Test Image View";
}

void CRightImageView::OnDraw(CDC* pDC)
{
	CRect rect;
	CRect rcClient;

	CimagealignDoc* doc = (CimagealignDoc*)GetDocument();

	// Get client window area in pixel dimensions
	GetClientRect(rcClient);

	// Get system defined colors
	COLORREF crLight = GetSysColor(COLOR_BTNHIGHLIGHT);
	COLORREF crShadow = GetSysColor(COLOR_BTNSHADOW);
	COLORREF crBtnFace = GetSysColor(COLOR_BTNFACE);
	pDC->SetBkMode(TRANSPARENT);
	CGdiObject *pOldFont = pDC->SelectStockObject(ANSI_VAR_FONT);
	rect = rcClient;

	// Draw title border and set title text
	rect.bottom = rect.top + BORDER_HEIGHT;
	rect.right = BORDER_LIMIT;
	pDC->FillSolidRect(rect, crBtnFace);
	pDC->Draw3dRect(rect, crLight, crShadow);
	pDC->DrawText(doc->testBorderTitle + doc->testFileName, 
		rect, DT_SINGLELINE | DT_VCENTER);

	// Fill client area as black
	rect.top = rect.bottom;
	rect.bottom = BORDER_LIMIT;
	pDC->FillSolidRect(rect, BLK);
	pDC->SelectObject(pOldFont);

	int width = doc->testWidth;
	int height = doc->testHeight;

	if (doc->testImage == 0)
		return;
	
	// Create device compatible bitmap
	CBitmap bitmap;
	bitmap.CreateCompatibleBitmap(pDC, width, height);

	/* Create compatible device context as storage to
	   blit image data from. */
	CDC cdc;
	cdc.CreateCompatibleDC(pDC);

	// Initialize bitmap with test image data and dimensions
	bitmap.SetBitmapBits(width * height * sizeof(DWORD), doc->testImage);

	CBitmap* oldBitmap = cdc.SelectObject(&bitmap);

	// Display bitmap in client window
	pDC->BitBlt(0, BORDER_HEIGHT, width, height, &cdc, 0, 0, SRCCOPY);

	pDC->SelectObject(oldBitmap);
}


// CRightImageView diagnostics

#ifdef _DEBUG
void CRightImageView::AssertValid() const
{
	CScrollView::AssertValid();
}

void CRightImageView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}
#endif //_DEBUG


// CRightImageView message handlers

