/***************************************************************************

PROJECT:			CS 297 Deliverable 3

FILE:				imagealigndoc.h

PURPOSE:			Document class definition to store all important 
					data used by the template, test, map, and feature 
					locate views.

COMPILER / TOOL:	Visual C++ .NET

PROGRAMMER:			Wallun Chan

START DATE:			4/1/05

***************************************************************************/

#pragma once

#include "edgefinder.h"
#include "utility.h"
#include "tps.h"

class CimagealignDoc : public CDocument
{
protected: // create from serialization only
	CimagealignDoc();
	DECLARE_DYNCREATE(CimagealignDoc)

// Attributes
public:
	// Detector for template view
	EdgeDetector* templateDetector;

	// Detector for test view
	EdgeDetector* testDetector;

	// Thin-plate-spline algorithm
	TPSSolver tps;

	// Points sampled from edges of template image
	VecIntPr templateSampledPts;

	// Points sampled from edges of test image
	VecIntPr testSampledPts;

	// Centroids of mapped user-selected points 
	VecIntPr centroids;

	// Coordinates of user-specified selection rectangle
	Boxes boxes;

	//
	PtGroup ptGroup;

	/* Used to track the upper-left and lower-right points
	   of a user-specified selection rectangle. */
	IntPr boxBoundary[2];

	// Pixel data of template image
	BYTE* templateImage;

	// Pixel data of test image
	BYTE* testImage;

	// Pixel data of feature located image
	BYTE* featureImage;

	// Pixel dimensions of template image
	int templateWidth;
	int templateHeight;

	// Pixel dimensions of test image
	int testWidth;
	int testHeight;

	// Pixel dimensions of feature located image
	int featureWidth;
	int featureHeight;

	/* Used to track the number of user clicks inside template
	   view area. */
	int pointCount;

	// Checks to see template and test images are mapped
	BOOL isMapped;

	// Checks to see if mapped points should be convex hulled
	BOOL showGrouping;

	// Checks to see if outliers should be removed from mapped points
	BOOL removeOutliers;

	// Border title text of template view
	CString tempBorderTitle;

	// Border title text of template view
	CString testBorderTitle;

	// File name of current image displayed in template view
	CString tempFileName;

	// File name of current image displayed in test view
	CString testFileName;

// Operations
public:

// Overrides
public:
	virtual BOOL OnNewDocument();
	CView* getView(CRuntimeClass* runtimeClass);

// Implementation
public:
	virtual ~CimagealignDoc();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
	DECLARE_MESSAGE_MAP()
};


