/*************************************************************************************

PROJECT:			CS 297 Deliverable 3

FILE:				convexhull.cpp

PURPOSE:			Implements the Chain Hull algorithm to calculate the convex
					hull of a set of points.

COMPILER / TOOL:	Visual C++ .NET

PROGRAMMER:			Wallun Chan

START DATE:			4/1/05

REMARKS:			This code was borrowed from 
					http://softsurfer.com/Archive/algorithm_0109/algorithm_0109.htm
					and adapted to work with this application.

*************************************************************************************/

#include "stdafx.h"
#include "convexhull.h"

int isLeft(IntPr P0, IntPr P1, IntPr P2)
/* PURPOSE:		Tests if a point is Left|On|Right of an infinite line.
	
   RECEIVES:	Three points P0, P1, and P2.

   RETURNS:		> 0 for P2 left of the line through P0 and P1.
				= 0 for P2 on the line.
				< 0 for P2 right of the line.
*/
{
    return (P1.first - P0.first) * (P2.second - P0.second) - 
		(P2.first - P0.first) * (P1.second - P0.second);
}

bool hullComp(IntPr element1, IntPr element2)
/* PURPOSE:		Used to compare the coordinates of two elements.

   RECEIVES:	Two elements to be compared.

   RETURNS:		True if x-coordinate of first element is less
				than the second element, or if the y-coordinates
				are the same, then return true if x-coordinate
				of first element is less than the second element.
				Otherwise, return false.
*/
{
	return (element1.first < element2.first || 
		    (element1.first == element2.first && 
			element1.second < element2.second));
}

VecIntPr convexHull(VecIntPr pts)
{	
	// hull will be used as the stack
	VecIntPr hull;
	if (pts.size() < 3)
	{	return pts;
	}
	hull.resize(pts.size());
	sort(pts.begin(), pts.end(), hullComp);
	int bot = 0;	// index for bottom and top of the stack
	int top = -1;	// index for top of the stack
CString s;
	// Get the indices of points with min x-coord and min|max y-coord
	int minmin = 0;
	int minmax;
	int xmin = pts[0].first;
	int n = (int)pts.size();
	int i;
	for (i = 1; i < n; i++)
	{	if (pts[i].first != xmin)
		{	break;
		}
	}
	minmax = i - 1;
	if (minmax == n - 1)	// degenerate case: all x-coords == xmin
	{	hull[++top] = pts[minmin];
		if (pts[minmax].second != pts[minmin].second)	// a nontrivial segment
		{    hull[++top] = pts[minmax];
		}
		hull[++top] = pts[minmin];		// add polygon endpoint
		hull.resize(top);
		return hull;
	}

	// Get the indices of points with max x-coord and min|max y-coord
    int maxmin;
	int maxmax = n - 1;
    int xmax = pts[n - 1].first;
    for (i = n - 2; i >= 0; i--)
	{   if (pts[i].first != xmax) 
		{	break;
		}
	}
	maxmin = i + 1;

	// Compute the lower hull on the stack H
	hull[++top] = pts[minmin];	// push minmin point onto stack
	i = minmax;
	while (++i <= maxmin)
	{	// the lower line joins P[minmin] with P[maxmin]
		if (isLeft(pts[minmin], pts[maxmin], pts[i]) >= 0 && i < maxmin)
		{	continue;	// ignore P[i] above or on the lower line
		}
		while (top > 0)		// there are at least 2 points on the stack
		{	// test if P[i] is left of the line at the stack top
			if (isLeft(hull[top - 1], hull[top], pts[i]) > 0)
			{	break;	// P[i] is a new hull vertex
			}
			else
			{	top--;	// pop top point off stack
			}
		}
		hull[++top] = pts[i];	// push P[i] onto stack
	}

	// Next, compute the upper hull on the stack H above the bottom hull
    if (maxmax != maxmin)	// if distinct xmax points
	{    hull[++top] = pts[maxmax];		// push maxmax point onto stack
	}
    bot = top;		// the bottom point of the upper hull stack
    i = maxmin;
	while (--i >= minmax)
	{	// the upper line joins P[maxmax] with P[minmax]
		if (isLeft(pts[maxmax], pts[minmax], pts[i]) >= 0 && i > minmax)
		{    continue;		// ignore P[i] below or on the upper line
		}
		while (top > bot)	// at least 2 points on the upper stack
		{	// test if P[i] is left of the line at the stack top
			if (isLeft(hull[top-1], hull[top], pts[i]) > 0)
			{	break;		// P[i] is a new hull vertex
			}
			else
			{	top--;		// pop top point off stack
			}
		}
		hull[++top] = pts[i];	// push P[i] onto stack
	}
    if (minmax != minmin)
	{    hull[++top] = pts[minmin];	// push joining endpoint onto stack
	}
	hull.resize(top);
    return hull;
}