/***************************************************************************

PROJECT:			CS 297 Deliverable 3

FILE:				edgefinder.h

PURPOSE:			Function declarations for Boie-Cox edge detector to
					perform edge detection on a grayscale image.

COMPILER / TOOL:	Visual C++ .NET

PROGRAMMER:			Wallun Chan

START DATE:			4/1/05

***************************************************************************/

#ifndef _EDGE_FINDER_H_
#define _EDGE_FINDER_H_

const int EDGE_X = 252;
const int EDGE_Y = 253;
const int EDGE_45 = 254;
const int EDGE_135 = 255;
const int FWD = 0;
const int BWD = 1;
const double diag_scale = 1.414;

class EdgeDetector
{
public:
	EdgeDetector(unsigned char* array = 0, int xdim = 0, int ydim = 0);
	~EdgeDetector();
	void freeImage();
	void setImage(unsigned char* array, int xdim, int ydim);
	unsigned char* image_find_edges(double sigma, int* threshold);
	unsigned char* image_hysteresis(int lo_thld);
	unsigned char* image_thin();
	unsigned char* get_image() const { return an_image; }
	unsigned char* get_edge_map() const { return edge_map; }
	unsigned char* get_edge_map_high() const { return edge_map_hi; }
	int get_x() const { return nx; }
	int get_y() const { return ny; }
protected:
	void image_detx();
	void image_dety();
	void image_det45();
	void image_det135();
	void generate_filter(double sigma);
	void image_convolve(unsigned char* pic);
	void image_neighbor_fwd(int iy, int ix, unsigned char* mapP);
	void image_neighbor_bwd(int iy, int ix, unsigned char* mapP);
	unsigned char* image_edges(int threshold);
	int image_locx(int iy, int ix, int index);
	int image_locy(int iy, int ix, int index);
	int image_loc45(int iy, int ix, int index);
	int image_loc135(int iy, int ix, int index);
	int image_zc_x(int ix, int iy, int i);
	int image_zc_y(int ix, int iy, int i);
	int image_zc_135(int ix, int iy, int i);
	int image_zc_45(int ix, int iy, int i);
	int image_edge_map_lo(int ix, int iy, int index);
	int Fcontour(int iy, int ix);
	int Bcontour(int iy, int ix);
	int image_threshold();
private:
	unsigned char* edge_map;
	unsigned char* edge_map_hi;
	unsigned char* an_image;
	int nx;
	int ny;
	int norm;
	int nf;
	int nf2;
	int orient_flag;
	int lo_threshold;
	int* gaussian;
	int* idx;
	int* idy;
	int* id45;
	int* id135;
	int* filter;
};

#endif